module ALU #(
  parameter BITS = 8
)(
  input                 i_clk,
  input                 i_rst,
  input                 i_eo,
  input                 i_e0,
  input                 i_e1,
  input                 i_e2,
  input                 i_ec,
  input      [BITS-1:0] i_rega,
  input      [BITS-1:0] i_regb,
  output     [BITS-1:0] o_bus,
  output reg [2:0]      o_flg,
  output reg [BITS-1:0] o_ans
);
reg [BITS:0] a_ans;
reg [2:0] a_flg;

assign o_bus = i_eo ? o_ans : {BITS{1'bz}};
always @(posedge i_clk) begin
  if (i_eo) begin
    o_flg <= a_flg;
  end
end

always @(*) begin
  if (i_rst)
    a_ans <= (BITS+1)'d0;
  else if (~i_clk) begin
    case ({i_e2,i_e1,i_e0})
      3'b011 : a_ans = i_rega + i_regb + i_ec;                    // A_PLUS_B
      3'b010 : begin                                              // A_MINUS_B
                 a_ans = i_rega - i_regb - !i_ec;
                 a_ans[BITS] = a_ans[BITS] ^ 1'b1;
               end
      3'b110 : a_ans = {1'b0, i_rega & i_regb};                   // A_AND_B
      3'b101 : a_ans = {1'b0, i_rega | i_regb};                   // A_OR_B
      3'b100 : a_ans = {1'b0, i_rega ^ i_regb};                   // A_XOR_B
      3'b001 : a_ans = i_regb - i_rega - i_ec;                    // NOT_B_MINUS_A
      default: a_ans = (BITS+1)'d0;
    endcase
    o_ans <= a_ans[BITS-1:0];
    a_flg[2] <= a_ans[BITS];
    a_flg[1] <= (a_ans[BITS-1:0]=={BITS{1'b0}});
    a_flg[0] <= a_ans[BITS];
  end
end
endmodule
